@@ -28,6 +28,8 @@ mod render;
 mod rules;
 mod tables;
 
+pub use grammar_files::lookup_package_json_for_path;
+
 lazy_static! {
     static ref JSON_COMMENT_REGEX: Regex = RegexBuilder::new("^\\s*//.*")
         .multi_line(true)
@@ -66,7 +68,11 @@ pub fn generate_parser_in_directory(
         }
     }
 
-    if repo_path.is_dir() && !repo_path.join("grammar.js").exists() && !path_in_ignore(&repo_path) {
+    let grammar_path = grammar_path
+        .map(PathBuf::from)
+        .unwrap_or(repo_path.join("grammar.js"));
+
+    if repo_path.is_dir() && !grammar_path.exists() && !path_in_ignore(&repo_path) {
         if let Some(dir_name) = repo_path
             .file_name()
             .map(|x| x.to_string_lossy().to_ascii_lowercase())
@@ -80,14 +86,8 @@ pub fn generate_parser_in_directory(
         }
     }
 
-    // Read the grammar.json.
-    let grammar_json = if let Some(path) = grammar_path {
-        load_grammar_file(path.as_ref(), js_runtime)?
-    } else {
-        let grammar_js_path =
-            grammar_path.map_or(repo_path.join("grammar.js"), std::convert::Into::into);
-        load_grammar_file(&grammar_js_path, js_runtime)?
-    };
+    // Read the grammar file.
+    let grammar_json = load_grammar_file(&grammar_path, js_runtime)?;
 
     let src_path = repo_path.join("src");
     let header_path = src_path.join("tree_sitter");
@@ -96,7 +96,7 @@ pub fn generate_parser_in_directory(
     fs::create_dir_all(&src_path)?;
     fs::create_dir_all(&header_path)?;
 
-    if grammar_path.is_none() {
+    if grammar_path.file_name().unwrap() != "grammar.json" {
         fs::write(src_path.join("grammar.json"), &grammar_json)
             .with_context(|| format!("Failed to write grammar.json to {src_path:?}"))?;
     }
@@ -116,7 +116,7 @@ pub fn generate_parser_in_directory(
     write_file(&header_path.join("array.h"), tree_sitter::ARRAY_HEADER)?;
     write_file(&header_path.join("parser.h"), tree_sitter::PARSER_HEADER)?;
 
-    if !path_in_ignore(&repo_path) {
+    if !path_in_ignore(&repo_path) && grammar_path == repo_path.join("grammar.js") {
         grammar_files::generate_grammar_files(&repo_path, &input_grammar.name, generate_bindings)?;
     }
 
@@ -187,42 +187,61 @@ pub fn load_grammar_file(grammar_path: &Path, js_runtime: Option<&str>) -> Resul
 fn load_js_grammar_file(grammar_path: &Path, js_runtime: Option<&str>) -> Result<String> {
     let grammar_path = fs::canonicalize(grammar_path)?;
 
+    #[cfg(windows)]
+    let grammar_path = url::Url::from_file_path(grammar_path)
+        .expect("Failed to convert path to URL")
+        .to_string();
+
     let js_runtime = js_runtime.unwrap_or("node");
 
-    let mut node_process = Command::new(js_runtime)
+    let mut js_command = Command::new(js_runtime);
+    match js_runtime {
+        "node" => {
+            js_command.args(["--input-type=module", "-"]);
+        }
+        "bun" => {
+            js_command.arg("-");
+        }
+        "deno" => {
+            js_command.args(["run", "--allow-all", "-"]);
+        }
+        _ => {}
+    }
+
+    let mut js_process = js_command
         .env("TREE_SITTER_GRAMMAR_PATH", grammar_path)
         .stdin(Stdio::piped())
         .stdout(Stdio::piped())
         .spawn()
         .with_context(|| format!("Failed to run `{js_runtime}`"))?;
 
-    let mut node_stdin = node_process
+    let mut js_stdin = js_process
         .stdin
         .take()
-        .with_context(|| "Failed to open stdin for node")?;
+        .with_context(|| format!("Failed to open stdin for {js_runtime}"))?;
     let cli_version = Version::parse(env!("CARGO_PKG_VERSION"))
         .with_context(|| "Could not parse this package's version as semver.")?;
     write!(
-        node_stdin,
-        "global.TREE_SITTER_CLI_VERSION_MAJOR = {};
-        global.TREE_SITTER_CLI_VERSION_MINOR = {};
-        global.TREE_SITTER_CLI_VERSION_PATCH = {};",
+        js_stdin,
+        "globalThis.TREE_SITTER_CLI_VERSION_MAJOR = {};
+         globalThis.TREE_SITTER_CLI_VERSION_MINOR = {};
+         globalThis.TREE_SITTER_CLI_VERSION_PATCH = {};",
         cli_version.major, cli_version.minor, cli_version.patch,
     )
-    .with_context(|| "Failed to write tree-sitter version to node's stdin")?;
-    let javascript_code = include_bytes!("./dsl.js");
-    node_stdin
-        .write(javascript_code)
-        .with_context(|| "Failed to write grammar dsl to node's stdin")?;
-    drop(node_stdin);
-    let output = node_process
+    .with_context(|| format!("Failed to write tree-sitter version to {js_runtime}'s stdin"))?;
+    js_stdin
+        .write(include_bytes!("./dsl.js"))
+        .with_context(|| format!("Failed to write grammar dsl to {js_runtime}'s stdin"))?;
+    drop(js_stdin);
+
+    let output = js_process
         .wait_with_output()
-        .with_context(|| "Failed to read output from node")?;
+        .with_context(|| format!("Failed to read output from {js_runtime}"))?;
     match output.status.code() {
-        None => panic!("Node process was killed"),
+        None => panic!("{js_runtime} process was killed"),
         Some(0) => {
-            let stdout =
-                String::from_utf8(output.stdout).with_context(|| "Got invalid UTF8 from node")?;
+            let stdout = String::from_utf8(output.stdout)
+                .with_context(|| format!("Got invalid UTF8 from {js_runtime}"))?;
 
             let mut grammar_json = &stdout[..];
 
@@ -244,7 +263,7 @@ fn load_js_grammar_file(grammar_path: &Path, js_runtime: Option<&str>) -> Result
             .with_context(|| "Failed to serialize grammar JSON")?
                 + "\n")
         }
-        Some(code) => Err(anyhow!("Node process exited with status {code}")),
+        Some(code) => Err(anyhow!("{js_runtime} process exited with status {code}")),
     }
 }
 
